@@ -5,6 +5,7 @@ from __future__ import absolute_import
 import os
 import multiprocessing
 import socket
+import sys
 import time
 import uuid
 
@@ -21,6 +22,10 @@ addressbook = thriftpy2.load(os.path.join(os.path.dirname(__file__),
                                           "addressbook.thrift"))
 
 
+if sys.platform == "win32":
+    pytest.skip("requires fork", allow_module_level=True)
+
+
 class Dispatcher():
     def __init__(self):
         self.ab = addressbook.AddressBook()
@@ -117,12 +122,23 @@ def client_context_with_url(timeout=3000):
                           url="http://127.0.0.1:6080", timeout=timeout)
 
 
+def client_context_with_malformed_path(timeout=3000):
+    return client_context(addressbook.AddressBookService, host="127.0.0.1",
+                          port=6080, path="foo", timeout=timeout)
+
+
 def client_with_url(timeout=3000):
     return make_client(addressbook.AddressBookService,
                        url="http://127.0.0.1:6080", timeout=timeout)
 
 
 def client_without_url(timeout=3000):
+    return make_client(addressbook.AddressBookService, host="127.0.0.1",
+                       port=6080, path="/foo", timeout=timeout)
+
+
+@pytest.fixture
+def client_with_malformed_path(timeout=3000):
     return make_client(addressbook.AddressBookService, host="127.0.0.1",
                        port=6080, path="foo", timeout=timeout)
 
@@ -157,8 +173,9 @@ def client_with_custom_header_factory(timeout=3000):
 
 
 def test_client_context(server):
-    with client() as c1, client_context_with_url() as c2:
-        assert c1.hello("world") == c2.hello("world")
+    with client() as c1, client_context_with_url() as c2,\
+         client_context_with_malformed_path() as c3:
+        assert c1.hello("world") == c2.hello("world") == c3.hello("world")
 
 
 def test_clients(server):
@@ -173,6 +190,10 @@ def test_clients_without_url(server):
     assert c.hello("world") == "hello world"
 
 
+def test_client_with_malformed_path(client_with_malformed_path):
+    assert client_with_malformed_path.hello("world") == "hello world"
+
+
 def test_client_context_with_header_factory(server):
     with client_context_with_header_factory() as c:
         assert c.hello("world") == "hello world"
