@@ -1,6 +1,9 @@
-# -*- coding: utf-8 -*-
+from __future__ import absolute_import, annotations
 
-from __future__ import absolute_import
+try:
+    from typing import Protocol
+except ImportError:
+    from typing_extensions import Protocol
 
 from ..thrift import TType, TException
 
@@ -20,6 +23,14 @@ def readall(read_fn, sz):
     return buff
 
 
+class TTransportFactory(Protocol):
+    """Transport factory interface for type annotations."""
+
+    def get_transport(self, trans) -> TTransportBase:
+        """Return a transport instance wrapping the given transport."""
+        ...
+
+
 class TTransportBase(object):
     """Base class for Thrift transport layer."""
 
